package it.mokabyte.pattern.dao;

import java.sql.*;
import javax.ejb.*;
import java.util.*;
import javax.sql.* ;
import javax.naming.* ;

import it.mokabyte.pattern.pk.AccountPK;
import it.mokabyte.pattern.valueobject.AccountOM;
import it.mokabyte.pattern.exception.DAOException;

/**
 * <p>Title: MyDaoImpl</p>
 * <p>Description: class that implements the data access logic to DBMS by JDBC</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: </p>
 * @author Rossini S. - Dozio L.
 * @version 1.0
 *
 * DBMS Table : MOKA_ACCOUNT<BR>
 * <pre>
 *    ID       (VARCHAR(255))
 *    NAME     (VARCHAR(255))
 *    ADDRESS  (VARCHAR(255))
 *    BALANCE  (DOUBLEPRECISION)
 * </pre>
 */
public class MyDaoImpl implements MyDao {

    /** The datasource */
    private DataSource dataSource;

    /**
     * Retrieve the datasource from JNDI using the given name.
     * @param dataSourceName the JNDI name of the DataSource to look up
     * @exception NamingException thrown for errors with JNDI
     */
    public MyDaoImpl(String dataSourceName) throws NamingException {
        InitialContext ic = new InitialContext();
        System.out.println("MyDaoImpl::MyDaoImpl : looking up for ["+dataSourceName+"] ...");
        dataSource = (DataSource)ic.lookup(dataSourceName);
        System.out.println("MyDaoImpl::MyDaoImpl : dataSource["+dataSource+"] intiliazed !");
    }

    /**
     * Insert a row into the MOKA_ACCOUNT table.
     * SQL : <code>INSERT INTO MOKA_ACCOUNT(ID, NAME, ADDRESS, BALANCE) VALUES (id,name,address,balance) </code>
     * @return the number of rows inserted
     * @param pk primary key for the account
     * @param name the account user name
     * @param address the account user address
     * @param balance the initial balance for this account
     * @exception DAOException thrown if a database error
     */
    public int insertAccount(AccountPK pk, String name, String address, double balance) throws DAOException {
        Connection con = null;
        PreparedStatement stmt = null;
        try {
            con = dataSource.getConnection();
            stmt = con.prepareStatement(
              "INSERT INTO MOKA_ACCOUNT(ID, NAME, ADDRESS, BALANCE) VALUES (?, ?, ?, ?)");
            stmt.setString(1, pk.id);
            stmt.setString(2, name);
            stmt.setString(3, address);
            stmt.setDouble(4, balance);
            int res = stmt.executeUpdate();
            System.out.println("MyDao.insertAccount : ennuple updated ["+res+"]...");
            return res;
        } catch(SQLException sqle) {
            System.out.println("MyDaoImpl.insertAccount : " + sqle);
            throw new DAOException(sqle.getMessage());
        } finally {
            closeResources(con, stmt, null);
        }
    }

    /**
     * Delete a row from the MOKA_ACCOUNT table.  Use the sql:
     * SQL : <code>DELETE FROM MOKA_ACCOUNT WHERE ID = 'ID'</code>
     * @return the number of rows deleted
     * @param  the primary key
     * @exception DAOException thrown if a database error
     */
    public int deleteAccount(AccountPK pk) throws DAOException {
        Connection con = null;
        PreparedStatement stmt = null;

        try {
            con = dataSource.getConnection();
            stmt = con.prepareStatement("DELETE FROM MOKA_ACCOUNT WHERE ID = ?");

            stmt.setString(1, pk.id);

            int res = stmt.executeUpdate();
            System.out.println("MyDao.deleteAccount : ennuple deleted ["+res+"]...");
            return res;
        } catch(SQLException sqle) {
            System.out.println("MyDaoImpl.deleteAccount : " + sqle);
            throw new DAOException(sqle.getMessage());
        }finally {
            closeResources(con, stmt, null);
        }
    }

    /**
     * Get a user from MOKA_ACCOUNT table.  Use the sql:
     * SQL : <code>SELECT NAME,ADDRESS,BALANCE FROM MOKA_ACCOUNT WHERE ID = 'ID'</code>
     * @return the ObjectModel of the Account
     * @param  the primary key
     * @exception DAOException thrown if a database error
     */
    public AccountOM readAccount(AccountPK pk) throws DAOException {

        AccountOM account = null;
        Connection con = null;
        PreparedStatement stmt = null;
        ResultSet rslt = null;

        try {
            con = dataSource.getConnection();
            System.out.println("MyDao.readAccount : ID["+pk.id+"] - con["+con+"]...");

            stmt = con.prepareStatement("SELECT NAME,ADDRESS,BALANCE FROM MOKA_ACCOUNT WHERE ID = ? ");
            stmt.setString(1, pk.id);
            rslt = stmt.executeQuery();

            if (rslt.next()) {
                String name = rslt.getString(1);
                String address = rslt.getString(2);
                double balance = rslt.getDouble(3);
                account = new AccountOM(pk.id,name,address,balance);
            }
            System.out.println("MyDao.readAccount : returning ["+account+"]...");
            return account;

        } catch(SQLException sqle) {
            System.out.println("MyDaoImpl.readAccount : " + sqle);
            throw new DAOException(sqle.getMessage());
        }finally {
            closeResources(con, stmt, null);
        }
    }


    /**
     * Update a row from the MOKA_ACCOUNT table.
     * SQL : <code>UPDATE MOKA_ACCOUNT SET BALANCE,NAME,ADDRESS = ... WHERE ID = <key></code>
     * @return the number of rows updated
     * @param key the primary key
     * @param name user name
     * @param address user address
     * @param balance user balance
     * @exception DAOException thrown if a database error
     */
    public int updateAccount(AccountPK key, String name, String address, double balance) throws DAOException {
        Connection con = null;
        PreparedStatement stmt = null;
        try {
            con = dataSource.getConnection();
            System.out.println("MyDao.updateAccount : con["+con+"]...");
            stmt = con.prepareStatement(
            "UPDATE MOKA_ACCOUNT SET BALANCE = ?,NAME = ?,ADDRESS= ? WHERE ID = ?");

            stmt.setDouble(1, balance);
            stmt.setString(2, name);
            stmt.setString(3, address);
            stmt.setString(4, key.id);

            int res = stmt.executeUpdate();
            System.out.println("MyDao.updateAccount : ennuple updated ["+res+"]...");
            return res;
       } catch(SQLException sqle) {
            System.out.println("MyDaoImpl.updateAccount : " + sqle);
            throw new DAOException(sqle.getMessage());
        } finally {
            closeResources(con, stmt, null);
        }
    }

    /**
     * @return Whether the indicated account exists
     * @param key the Primary Key for this account
     * @exception DAOException thrown if a database error
     */
     public boolean accountExists(AccountPK key) throws DAOException {
        return (readAccount(key) != null);
    }

    /**
     * Close all Database-Resources passed in.  <code>null</code> is an acceptable
     * value for all arguments.
     * @param con The Connection to close
     * @param stmt The PreparedStatement to close
     * @param rslt The ResultSet to close
     */
    private void closeResources(Connection con, PreparedStatement stmt, ResultSet rslt) {

         if (rslt != null) {
            try {
                rslt.close();
            } catch (SQLException e) {
            }
        }
        if (stmt != null) {
            try {
                stmt.close();
            } catch (SQLException e) {
            }
        }
        if (con != null) {
            try {
                con.close();
            } catch (SQLException e) {
            }
        }
    }
}